use actix_web::{error::ErrorInternalServerError, web, HttpResponse, Responder};
use argon2::{
    password_hash::{rand_core::OsRng, PasswordHasher, SaltString},
    Argon2,
};
use entity::users;
use sea_orm::{
    entity::prelude::DateTime, prelude::Uuid, ActiveModelTrait, ActiveValue, EntityTrait,
};
use serde::{Deserialize, Serialize};

use crate::AppState;

#[derive(Serialize)]
pub struct UserWithoutPassword {
    id: Uuid,
    username: String,
    name: String,
    email: String,
    created_at: DateTime,
    updated_at: DateTime,
}

#[derive(Deserialize)]
pub struct CreateUserDto {
    username: String,
    name: String,
    email: String,
    password: String,
}

#[derive(Deserialize)]
pub struct UpdateUserDto {
    username: String,
    name: String,
    email: String,
}

impl From<users::Model> for UserWithoutPassword {
    fn from(value: users::Model) -> Self {
        Self {
            id: value.user_id,
            username: value.username,
            name: value.name,
            email: value.email,
            created_at: value.created_at,
            updated_at: value.updated_at,
        }
    }
}

pub async fn get_users(state: web::Data<AppState>) -> actix_web::Result<impl Responder> {
    let db = &state.db;

    let result = users::Entity::find()
        .all(db)
        .await
        .map_err(ErrorInternalServerError)?;

    Ok(web::Json(result))
}

pub async fn create_user(
    state: web::Data<AppState>,
    user: web::Json<CreateUserDto>,
) -> actix_web::Result<impl Responder> {
    let db = &state.db;
    let user = user.into_inner();

    let salt = SaltString::generate(&mut OsRng);
    let argon2 = Argon2::default();

    let password_hash = argon2
        .hash_password(user.password.as_bytes(), &salt)
        .map_err(ErrorInternalServerError)?;

    let user = users::ActiveModel {
        user_id: ActiveValue::NotSet,
        username: ActiveValue::Set(user.username),
        name: ActiveValue::Set(user.name),
        email: ActiveValue::Set(user.email),
        password_hash: ActiveValue::Set(password_hash.to_string()),
        created_at: ActiveValue::NotSet,
        updated_at: ActiveValue::NotSet,
    };

    let user = user.insert(db).await.map_err(ErrorInternalServerError)?;

    Ok(web::Json(UserWithoutPassword::from(user)))
}

pub async fn delete_user(
    state: web::Data<AppState>,
    path: web::Path<Uuid>,
) -> actix_web::Result<impl Responder> {
    let id = path.into_inner();

    let db = &state.db;

    entity::users::Entity::delete_by_id(id)
        .exec(db)
        .await
        .map_err(ErrorInternalServerError)?;

    Ok(HttpResponse::Ok().finish())
}

pub async fn update_user(
    state: web::Data<AppState>,
    path: web::Path<Uuid>,
    user: web::Json<UpdateUserDto>,
) -> actix_web::Result<impl Responder> {
    let id = path.into_inner();

    let db = &state.db;

    let user = user.into_inner();

    let user = users::ActiveModel {
        user_id: ActiveValue::Unchanged(id),
        username: ActiveValue::Set(user.username),
        name: ActiveValue::Set(user.name),
        email: ActiveValue::Set(user.email),
        password_hash: ActiveValue::NotSet,
        created_at: ActiveValue::NotSet,
        updated_at: ActiveValue::NotSet,
    };

    let res = user.update(db).await.map_err(ErrorInternalServerError)?;

    Ok(web::Json(res))
}
